// Craig S. Kaplan
// A complicated example of hierarchical modelling --
// draw a street of houses with a night sky and some
// stars.

// TODO: decide on appropriate naming for function window() whereas that name is already taken in JS

function doorknob()
{
  fill( 140 );
  ellipse( 0, 0, 100, 100 );
  fill( 80 );
  ellipse( 0, 0, 50, 50 );
}

function door()
{
  fill( 85, 58, 3 );
  rect( -25, -50, 50, 100 );
  fill( 113, 77, 5 );
  rect( -20, -45, 40, 40 );
  rect( -20, 5, 40, 40 );

  push();
  translate( 20, 0 );
  scale( 0.1 );
  doorknob();
  pop();
}

function window_()
{
  fill( 85, 58, 3 );
  rect( -25, -30, 50, 60 );
  fill( 255, 249, 126 );
  rect( -20, -25, 40, 50 );
  fill( 100 );
  rect( -20, -25, 40, 30 );
  line( 0, 5, 0, 15 );
  fill( 0 );
  ellipse( 0, 15, 4, 4 );
}

function house()
{
  // The body and roof of the house.
  fill( 214, 206, 160 );
  rect( -100, -150, 200, 150 );
  fill( 103, 101, 88 );
  triangle( 0, -200, -120, -150, 120, -150 );

  // Add in the two doors. Note that one door
  // is the mirror image of the other one, hence
  // the "scale(-1,1)".
  push();
  scale( 0.5 );
  translate( -25, -50 );
  door();
  pop();
  push();
  scale( 0.5 );
  scale( -1, 1 );
  translate( -25, -50 );
  door();
  pop();

  // Draw a row of three windows.
  push();
  translate( -60, -100 );
  for ( let idx = 0; idx < 3; ++idx ) {
    window_();
    translate( 60, 0 );
  }
  pop();
}

function star()
{
  fill( 255 );
  noStroke();
  beginShape();
  // A sneaky trick for drawing a five-pointed star.
  for( let idx = 0; idx < 5; ++idx ) {
    vertex( 50*cos(TWO_PI*0.4*idx), 50*sin(TWO_PI*0.4*idx) );
  }
  endShape();
  stroke( 0 );
}

function setup()
{
  createCanvas( 800, 600 );
  
  // Set the initial colour to a night sky.
  background( 26, 21, 72 );
  stroke( 0 );
  strokeWeight( 1.0 );

  // Draw a bunch of random stars.
  for( let idx = 0; idx < 200; ++idx ) {
    push();
    translate( random( width ), random( 400 ) );
    rotate( random( TWO_PI ) );
    scale( random( 0.01, 0.1 ) );
    star();
    pop();
  }

  push();
  translate( 0, 400 );
  scale( 0.3, 0.3 );

  // Draw a row of houses.
  for ( let idx = 0; idx < 11; ++idx ) {
    house();
    translate( 270, 0 );
  }
  pop();
  
  // Draw the street.
  fill( 180 );
  rect( 0, 400, width, height-400 );

}
