// Craig S. Kaplan

// Progressively draw a random snowflake by drawing random
// black and white triangles and all their symmetric copies.
// Press a key to add a new triangle.

// It's tempting to put the calls to random() in here.  But
// if you do that, then every individual "copy" of the triangle
// will in fact be different from the others.  We'll come back
// to this topic a bit later in the term.  Instead, we expect
// an array of the coordinates of a triangle to be passed down
// to us.
function drawHalfArm( cs ) {
  triangle( cs[0].x, cs[0].y,
            cs[1].x, cs[1].y,
            cs[2].x, cs[2].y );
}

// Draw a triangle together with its mirror image.
function drawSymmetricArm( tri ) {
  push();
  drawHalfArm( tri );
  scale( -1.0, 1.0 );
  drawHalfArm( tri );
  pop();
}

// Draw six copies of a triangle with its mirror image.
function drawAllArms( tri ) {
  push();
  for( let idx = 0; idx < 6; ++idx ) {
    drawSymmetricArm( tri );
    rotate( radians( 60 ) );
  }
  pop();
}

function setup() {
  createCanvas( 500, 500 );
  
  background( 0 );
  fill( 255 );
  noStroke();
}

function keyPressed() { 
  let tri = [
    createVector(random( 100 ), random( 200 )), 
    createVector(random( 100 ), random( 200 )),
    createVector(random( 100 ), random( 200 )) ];
    
  if( random( 1 ) < 0.25 ) {
    fill( 0 );
  } else {
    fill( 255 );
  }
    
  push();
  translate( 250, 250 );
  drawAllArms( tri );
  pop();
}