// Craig S. Kaplan

// A very explicit house drawing sketch that exposes
// (something like) the inner workings of geometric
// context, at least the translation part.  You don't
// actually need to write code like this -- with the
// geometric context functions (like translate()), 
// this stuff is all built-in.  But it's worth offering
// this demonstration.

function setup() {
  createCanvas( 500, 500 );
}

let tx = 0;
let ty = 0;

function myRect( x, y, w, h ) {
  rect( x + tx, y + ty, w, h );
}

function myEllipse( x, y, w, h ) {
  ellipse( x + tx, y + ty, w, h );
}

function myTriangle( x1, y1, x2, y2, x3, y3 ) {
  triangle( x1 + tx, y1 + ty, 
    x2 + tx, y2 + ty,
    x3 + tx, y3 + ty );
}

function drawHouse() {
  fill( 191, 179, 117 );
  // Facade
  myRect( 150, 200, 200, 150 );
  // Chimney
  myRect( 320, 120, 30, 80 );
  
  fill( 62, 54, 47 );
  // Roof
  myTriangle( 250, 100, 120, 200, 370, 200 );
  
  fill( 222, 88, 76 );
  // Door
  myRect( 230, 280, 40, 70 );
  
  fill( 232, 232, 232 );
  // Smoke
  myEllipse( 330, 100, 50, 20 );
  myEllipse( 350, 70, 40, 20 );
}

function draw() {
  background( 255 );
  drawHouse();
}

function mouseClicked() {
    tx = mouseX;
    ty = mouseY;
}